@@ -11,17 +11,18 @@ from threading import Lock
 from easyprocess import EasyProcess, EasyProcessError
 
 from pyvirtualdisplay import xauth
-from pyvirtualdisplay.util import get_helptext, py2
+from pyvirtualdisplay.util import get_helptext, platform_is_osx
+
+log = logging.getLogger(__name__)
 
 # try:
 #     import fcntl
 # except ImportError:
 #     fcntl = None
 
-
 _mutex = Lock()
+_mutex_popen = Lock()
 
-log = logging.getLogger(__name__)
 
 _MIN_DISPLAY_NR = 1000
 _USED_DISPLAY_NR_LIST = []
@@ -53,7 +54,7 @@ def _lock_files():
     return ls
 
 
-def _search_for_display(randomizer=None):
+def _search_for_display():
     # search for free display
     ls = list(map(lambda x: int(x.split("X")[1].split("-")[0]), _lock_files()))
     if len(ls):
@@ -61,9 +62,6 @@ def _search_for_display(randomizer=None):
     else:
         display = _MIN_DISPLAY_NR
 
-    if randomizer:
-        display = randomizer.generate()
-
     return display
 
 
@@ -72,28 +70,35 @@ class AbstractDisplay(object):
     Common parent for X servers (Xvfb,Xephyr,Xvnc)
     """
 
-    def __init__(self, program, use_xauth, randomizer, retries, extra_args):
+    def __init__(self, program, use_xauth, retries, extra_args, manage_global_env):
         self._extra_args = extra_args
         self._retries = retries
         self._program = program
-        self._randomizer = randomizer
         self.stdout = None
         self.stderr = None
         self.old_display_var = None
         self._subproc = None
         self.display = None
-        self.is_started = False
+        self._is_started = False
+        self._manage_global_env = manage_global_env
+        self._reset_global_env = False
+        self._pipe_wfd = None
+        self._retries_current = 0
 
         helptext = get_helptext(program)
         self._has_displayfd = "-displayfd" in helptext
         if not self._has_displayfd:
             log.debug("-displayfd flag is missing.")
-        # if check_startup and not has_displayfd:
-        #     check_startup = False
-        #     log.warning(
-        #         program
-        #         + " -displayfd flag is not supported, 'check_startup' parameter has been disabled"
-        #     )
+        PYVIRTUALDISPLAY_DISPLAYFD = os.environ.get("PYVIRTUALDISPLAY_DISPLAYFD")
+        if PYVIRTUALDISPLAY_DISPLAYFD:
+            log.debug("PYVIRTUALDISPLAY_DISPLAYFD=%s", PYVIRTUALDISPLAY_DISPLAYFD)
+            # '0'->false, '1'->true
+            self._has_displayfd = bool(int(PYVIRTUALDISPLAY_DISPLAYFD))
+        else:
+            # TODO: macos: displayfd is available on XQuartz-2.7.11 but it doesn't work, always 0 is returned
+            if platform_is_osx():
+                self._has_displayfd = False
+
         self._check_flags(helptext)
 
         if use_xauth and not xauth.is_installed():
@@ -102,22 +107,6 @@ class AbstractDisplay(object):
         self._use_xauth = use_xauth
         self._old_xauth = None
         self._xauth_filename = None
-        # self.check_startup = check_startup
-        # if check_startup and not fcntl:
-        #     self.check_startup = False
-        #     log.warning(
-        #         "fcntl module can't be imported, 'check_startup' parameter has been disabled"
-        #     )
-        #     log.warning("fnctl module does not exist on Windows")
-        # if self.check_startup:
-        #     rp, wp = os.pipe()
-        #     fcntl.fcntl(rp, fcntl.F_SETFD, fcntl.FD_CLOEXEC)
-        #     # TODO: to properly allow to inherit fds to subprocess on
-        #     # python 3.2+ the easyprocess needs small fix..
-        #     fcntl.fcntl(wp, fcntl.F_SETFD, 0)
-        #     self.check_startup_fd = wp
-        #     self._check_startup_fd = rp
-        # self.proc = EasyProcess(self._cmd())
 
     def _check_flags(self, helptext):
         pass
@@ -141,24 +130,30 @@ class AbstractDisplay(object):
             except KeyError:
                 log.warning("$DISPLAY was already unset.")
         else:
-            log.debug("$DISPLAY=%s", d)
+            log.debug("set $DISPLAY=%s", d)
             os.environ["DISPLAY"] = d
 
+    def _env(self):
+        env = os.environ.copy()
+        env["DISPLAY"] = self.new_display_var
+        return env
+
     def start(self):
         """
         start display
 
         :rtype: self
         """
-        if self.is_started:
+        if self._is_started:
             raise XStartError(self, "Display was started twice.")
-        self.is_started = True
+        self._is_started = True
 
         if self._has_displayfd:
-            self._start1()
+            self._start1_has_displayfd()
         else:
             i = 0
             while True:
+                self._retries_current = i + 1
                 try:
                     self._start1()
                     break
@@ -171,50 +166,64 @@ class AbstractDisplay(object):
                             "No success after %s retries. Last stderr: %s"
                             % (self._retries, self.stderr)
                         )
-                finally:
-                    self._redirect_display(False)
-        self._redirect_display(True)
+        if self._manage_global_env:
+            self._redirect_display(True)
+            self._reset_global_env = True
 
-    def _start1(self):
-        if self._has_displayfd:
-            # stdout doesn't work on osx -> create own pipe
-            rfd, self._pipe_wfd = os.pipe()
-        else:
-            with _mutex:
-                self.display = _search_for_display(randomizer=self._randomizer)
-                while self.display in _USED_DISPLAY_NR_LIST:
-                    self.display += 1
-                self.new_display_var = ":%s" % int(self.display)
+    def _popen(self, use_pass_fds):
+        with _mutex_popen:
+            if use_pass_fds:
+                self._subproc = subprocess.Popen(
+                    self._command,
+                    pass_fds=[self._pipe_wfd],
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    shell=False,
+                )
+            else:
+                self._subproc = subprocess.Popen(
+                    self._command,
+                    stdout=subprocess.PIPE,
+                    stderr=subprocess.PIPE,
+                    shell=False,
+                )
 
-                _USED_DISPLAY_NR_LIST.append(self.display)
+    def _start1_has_displayfd(self):
+        # stdout doesn't work on osx -> create own pipe
+        rfd, self._pipe_wfd = os.pipe()
 
         self._command = self._cmd() + self._extra_args
         log.debug("command: %s", self._command)
 
-        self._stdout_file = tempfile.TemporaryFile(prefix="stdout_")
-        self._stderr_file = tempfile.TemporaryFile(prefix="stderr_")
+        self._popen(use_pass_fds=True)
+
+        self.display = int(self._wait_for_pipe_text(rfd))
+        os.close(rfd)
+        os.close(self._pipe_wfd)
+
+        self.new_display_var = ":%s" % int(self.display)
+
+        if self._use_xauth:
+            self._setup_xauth()
+
+        # https://github.com/ponty/PyVirtualDisplay/issues/2
+        # https://github.com/ponty/PyVirtualDisplay/issues/14
+        self.old_display_var = os.environ.get("DISPLAY", None)
+
+    def _start1(self):
+        with _mutex:
+            self.display = _search_for_display()
+            while self.display in _USED_DISPLAY_NR_LIST:
+                self.display += 1
+            self.new_display_var = ":%s" % int(self.display)
+
+            _USED_DISPLAY_NR_LIST.append(self.display)
+
+        self._command = self._cmd() + self._extra_args
+        log.debug("command: %s", self._command)
+
+        self._popen(use_pass_fds=False)
 
-        if py2() or not self._has_displayfd:
-            self._subproc = subprocess.Popen(
-                self._command,
-                stdout=self._stdout_file,
-                stderr=self._stderr_file,
-                shell=False,
-            )
-        else:
-            if self._has_displayfd:
-                self._subproc = subprocess.Popen(
-                    self._command,
-                    pass_fds=[self._pipe_wfd],
-                    stdout=self._stdout_file,
-                    stderr=self._stderr_file,
-                    shell=False,
-                )
-        if self._has_displayfd:
-            # rfd = self.subproc.stdout.fileno()
-            self.display = int(self._wait_for_pipe_text(rfd))
-            os.close(rfd)
-            os.close(self._pipe_wfd)
         self.new_display_var = ":%s" % int(self.display)
 
         if self._use_xauth:
@@ -226,65 +235,44 @@ class AbstractDisplay(object):
 
         # wait until X server is active
         start_time = time.time()
-        # if self.check_startup:
-        #     rp = self._check_startup_fd
-        #     display_check = None
-        #     rlist, wlist, xlist = select.select((rp,), (), (), _X_START_TIMEOUT)
-        #     if rlist:
-        #         display_check = os.read(rp, 10).rstrip()
-        #     else:
-        #         msg = "No display number returned by X server"
-        #         raise XStartTimeoutError(msg)
-        #     dnbs = str(self.display)
-        #     if bytes != str:
-        #         dnbs = bytes(dnbs, "ascii")
-        #     if display_check != dnbs:
-        #         msg = 'Display number "%s" not returned by X server' + str(
-        #             display_check
-        #         )
-        #         raise XStartTimeoutError(msg % self.display)
 
-        if not self._has_displayfd:
-            self._redirect_display(True)  # for xdpyinfo
-            d = self.new_display_var
-            ok = False
-            time.sleep(0.05)  # give time for early exit
-            while True:
-                if not self.is_alive():
-                    break
-
-                try:
-                    xdpyinfo = EasyProcess(["xdpyinfo"])
-                    xdpyinfo.enable_stdout_log = False
-                    xdpyinfo.enable_stderr_log = False
-                    exit_code = xdpyinfo.call().return_code
-                except EasyProcessError:
-                    log.warning(
-                        "xdpyinfo was not found, X start can not be checked! Please install xdpyinfo!"
-                    )
-                    time.sleep(_X_START_WAIT)  # old method
-                    ok = True
-                    break
-
-                if exit_code != 0:
-                    pass
-                else:
-                    log.info('Successfully started X with display "%s".', d)
-                    ok = True
-                    break
-
-                if time.time() - start_time >= _X_START_TIMEOUT:
-                    break
-                time.sleep(_X_START_TIME_STEP)
+        d = self.new_display_var
+        ok = False
+        time.sleep(0.05)  # give time for early exit
+        while True:
             if not self.is_alive():
-                log.warning("process exited early. stderr:%s", self.stderr)
-                msg = "Failed to start process: %s"
-                raise XStartError(msg % self)
-            if not ok:
-                msg = 'Failed to start X on display "%s" (xdpyinfo check failed, stderr:[%s]).'
-                raise XStartTimeoutError(msg % (d, xdpyinfo.stderr))
+                break
 
-        return self
+            try:
+                xdpyinfo = EasyProcess(["xdpyinfo"], env=self._env())
+                xdpyinfo.enable_stdout_log = False
+                xdpyinfo.enable_stderr_log = False
+                exit_code = xdpyinfo.call().return_code
+            except EasyProcessError:
+                log.warning(
+                    "xdpyinfo was not found, X start can not be checked! Please install xdpyinfo!"
+                )
+                time.sleep(_X_START_WAIT)  # old method
+                ok = True
+                break
+
+            if exit_code != 0:
+                pass
+            else:
+                log.info('Successfully started X with display "%s".', d)
+                ok = True
+                break
+
+            if time.time() - start_time >= _X_START_TIMEOUT:
+                break
+            time.sleep(_X_START_TIME_STEP)
+        if not self.is_alive():
+            log.warning("process exited early. stderr:%s", self.stderr)
+            msg = "Failed to start process: %s"
+            raise XStartError(msg % self)
+        if not ok:
+            msg = 'Failed to start X on display "%s" (xdpyinfo check failed, stderr:[%s]).'
+            raise XStartTimeoutError(msg % (d, xdpyinfo.stderr))
 
     def _wait_for_pipe_text(self, rfd):
         s = ""
@@ -314,10 +302,11 @@ class AbstractDisplay(object):
 
         :rtype: self
         """
-        if not self.is_started:
+        if not self._is_started:
             raise XStartError("stop() is called before start().")
 
-        self._redirect_display(False)
+        if self._reset_global_env:
+            self._redirect_display(False)
 
         if self.is_alive():
             try:
@@ -336,13 +325,7 @@ class AbstractDisplay(object):
 
     def _read_stdout_stderr(self):
         if self.stdout is None:
-            self._stdout_file.seek(0)
-            self._stderr_file.seek(0)
-            self.stdout = self._stdout_file.read()
-            self.stderr = self._stderr_file.read()
-
-            self._stdout_file.close()
-            self._stderr_file.close()
+            (self.stdout, self.stderr) = self._subproc.communicate()
 
             log.debug("stdout=%s", self.stdout)
             log.debug("stderr=%s", self.stderr)
@@ -387,10 +370,14 @@ class AbstractDisplay(object):
         self.stop()
 
     def is_alive(self):
+        if not self._subproc:
+            return False
         return self.return_code is None
 
     @property
     def return_code(self):
+        if not self._subproc:
+            return None
         rc = self._subproc.poll()
         if rc is not None:
             # proc exited
