@@ -15,6 +15,8 @@ class Display(object):
     :param visible: True -> Xephyr, False -> Xvfb
     :param backend: 'xvfb', 'xvnc' or 'xephyr', ignores ``visible``
     :param xauth: If a Xauthority file should be created.
+    :param manage_global_env: if True then $DISPLAY is set in os.environ 
+        which is not thread-safe. Use False to make it thread-safe.
     """
 
     def __init__(
@@ -26,9 +28,9 @@ class Display(object):
         bgcolor="black",
         use_xauth=False,
         # check_startup=False,
-        randomizer=None,
         retries=10,
         extra_args=[],
+        manage_global_env=True,
         **kwargs
     ):
         self._color_depth = color_depth
@@ -51,10 +53,11 @@ class Display(object):
             size=size,
             color_depth=color_depth,
             bgcolor=bgcolor,
-            randomizer=randomizer,
+            retries=retries,
             use_xauth=use_xauth,
             # check_startup=check_startup,
             extra_args=extra_args,
+            manage_global_env=manage_global_env,
             **kwargs
         )
 
@@ -101,14 +104,16 @@ class Display(object):
         """
         return self._obj.pid
 
-    @property
-    def is_started(self):
-        return self._obj.is_started
-
     @property
     def display(self):
+        """The new $DISPLAY variable as int.  Example 1 if $DISPLAY=':1'  """
         return self._obj.display
 
     @property
     def new_display_var(self):
+        """The new $DISPLAY variable like ':1'  """
         return self._obj.new_display_var
+
+    def env(self):
+        """env() copies global os.environ and adds disp.new_display_var"""
+        return self._obj._env()
